@@ -12,13 +12,14 @@ class MailboxHandler extends PFAHandler {
 
     # init $this->struct, $this->db_table and $this->id_field
     protected function initStruct() {
-        $passwordReset = Config::read('forgotten_user_password_reset');
+        $passwordReset = (int) Config::bool('forgotten_user_password_reset');
         $reset_by_sms = 0;
-        if ($passwordReset && Config::read('sms_send_function')) {
+        if ($passwordReset && Config::read_string('sms_send_function')) {
             $reset_by_sms = 1;
         }
 
-        $this->struct=array(
+        $this->struct = array(
+
             # field name                allow       display in...   type    $PALANG label                     $PALANG description                 default / options / ...
             #                           editing?    form    list
             'username'         => pacol($this->new, 1,      1,      'mail', 'pEdit_mailbox_username'        , ''                                , '' ),
@@ -30,7 +31,7 @@ class MailboxHandler extends PFAHandler {
             'maildir'          => pacol($this->new, 0,      1,      'text', ''                              , ''                                , '' ),
             'password'         => pacol(1,          1,      0,      'pass', 'password'                      , 'pCreate_mailbox_password_text'   , '' ),
             'password2'        => pacol(1,          1,      0,      'pass', 'password_again'                , ''                                 , '',
-                /*options*/ '',
+                /*options*/ array(),
                 /*not_in_db*/ 0,
                 /*dont_write_to_db*/ 1,
                 /*select*/ 'password as password2'
@@ -41,7 +42,7 @@ class MailboxHandler extends PFAHandler {
             # TODO: read used quota from quota/quota2 table
             'active'           => pacol(1,          1,      1,      'bool', 'active'                        , ''                                 , 1 ),
             'welcome_mail'     => pacol($this->new, $this->new, 0,  'bool', 'pCreate_mailbox_mail'          , ''                                 , 1,
-                /*options*/ '',
+                /*options*/ array(),
                 /*not_in_db*/ 1             ),
             'phone'            => pacol(1,  $reset_by_sms,  0,      'text', 'pCreate_mailbox_phone'         , 'pCreate_mailbox_phone_desc'       , ''),
             'email_other'      => pacol(1,  $passwordReset, 0,      'mail', 'pCreate_mailbox_email'         , 'pCreate_mailbox_email_desc'       , ''),
@@ -49,6 +50,7 @@ class MailboxHandler extends PFAHandler {
             'token_validity'   => pacol(1,          0,      0,      'ts',   ''                              , '', date("Y-m-d H:i:s",time())),
             'created'          => pacol(0,          0,      1,      'ts',   'created'                       , ''                                 ),
             'modified'         => pacol(0,          0,      1,      'ts',   'last_modified'                 , ''                                 ),
+            'password_expiry'  => pacol(0,          0,      1,      'ts',   'password_expiration'           , ''                                 ),
             # TODO: add virtual 'notified' column and allow to display who received a vacation response?
         );
 
@@ -58,7 +60,7 @@ class MailboxHandler extends PFAHandler {
         }
     }
 
-    public function init($id) {
+    public function init($id) : bool {
         if (!parent::init($id)) {
             return false;
         }
@@ -96,7 +98,7 @@ class MailboxHandler extends PFAHandler {
         # } elseif ($maxquota < 0) {
             # TODO: show 'disabled' - at the moment, just shows '-1'
         } else {
-            $this->struct['quota']['desc'] = Config::lang_f('mb_max', $maxquota);
+            $this->struct['quota']['desc'] = Config::lang_f('mb_max', "" . $maxquota);
         }
     }
 
@@ -205,7 +207,7 @@ class MailboxHandler extends PFAHandler {
         foreach ($db_result as $key => $row) {
             if (isset($row['quota']) && is_numeric($row['quota']) && $row['quota'] > -1) { # quota could be disabled in $struct
                 $db_result[$key]['quotabytes'] = $row['quota'];
-                $db_result[$key]['quota'] = divide_quota($row['quota']); # convert quota to MB
+                $db_result[$key]['quota'] = divide_quota( (int) $row['quota']); # convert quota to MB
             } else {
                 $db_result[$key]['quotabytes'] = -1;
                 $db_result[$key]['quota'] = -1;
@@ -215,16 +217,20 @@ class MailboxHandler extends PFAHandler {
     }
 
 
-    protected function beforestore() {
-
-        if (isset($this->values['quota']) && $this->values['quota'] != -1) {
-            $multiplier = Config::read('quota_multiplier');
+    protected function preSave() : bool {
+        if (isset($this->values['quota']) && $this->values['quota'] != -1 && is_numeric($this->values['quota'])) {
+            $multiplier = Config::read_string('quota_multiplier');
             if ($multiplier == 0 || !is_numeric($multiplier)) { // or empty string, or null, or false...
                 $multiplier = 1;
             }
             $this->values['quota'] = $this->values['quota'] * $multiplier; # convert quota from MB to bytes
         }
 
+        // Avoid trying to store '' in an integer field
+        if ($this->values['quota'] === '') {
+            $this->values['quota'] = 0;
+        }
+
         $ah = new AliasHandler($this->new, $this->admin_username);
 
         $ah->calledBy('MailboxHandler');
@@ -250,15 +256,43 @@ class MailboxHandler extends PFAHandler {
             return false;
         }
 
-        if (!$ah->store()) {
+        if (!$ah->save()) {
             $this->errormsg[] = $ah->errormsg[0];
             return false;
         }
 
-        return true; # still here? good!
+        if (!empty($this->values['password'])) {
+            // provide some default value to keep MySQL etc happy.
+            $this->values['password_expiry'] = date('Y-m-d H:i', strtotime("+365 days"));
+            if (Config::bool('password_expiration')) {
+                $domain_dirty = $this->domain_from_id();
+                $domain = trim($domain_dirty, "`'"); // naive assumption it is ' escaping.
+                $password_expiration_value = (int)get_password_expiration_value($domain);
+                $this->values['password_expiry'] = date('Y-m-d H:i', strtotime("+$password_expiration_value day"));
+            }
+        }
+
+        return true;
     }
 
-    protected function storemore() {
+    protected function setmore(array $values) {
+        if (array_key_exists('quota', $this->values)) {
+            $this->values['quota'] = (int)$this->values['quota'];
+        }
+    }
+
+    // Could perhaps also use _validate_local_part($new_value) { .... }
+    public function set(array $values) {
+        // See: https://github.com/postfixadmin/postfixadmin/issues/282 - ensure the 'local_part' does not contain an @ sign.
+        $ok = true;
+        if (isset($values['local_part']) && strpos($values['local_part'], '@')) {
+            $this->errormsg['local_part'] = Config::lang('pCreate_mailbox_local_part_error');
+            $ok = false;
+        }
+        return $ok && parent::set($values);
+    }
+
+    protected function postSave() : bool {
         if ($this->new) {
             if (!$this->mailbox_post_script()) {
                 # return false; # TODO: should this be fatal?
@@ -323,7 +357,7 @@ class MailboxHandler extends PFAHandler {
         db_delete($this->db_table,          $this->id_field, $this->id); # finally delete the mailbox
 
         if (!$this->mailbox_postdeletion()) {
-            $this->error_msg[] = Config::Lang('mailbox_postdel_failed');
+            $this->errormsg[] = Config::Lang('mailbox_postdel_failed');
         }
 
         list(/*NULL*/, $domain) = explode('@', $this->id);
@@ -413,7 +447,7 @@ class MailboxHandler extends PFAHandler {
 
         $maildir_name_hook = Config::read('maildir_name_hook');
 
-        if ($maildir_name_hook != 'NO' && function_exists($maildir_name_hook)) {
+        if (is_string($maildir_name_hook) && $maildir_name_hook != 'NO' && function_exists($maildir_name_hook)) {
             $maildir = $maildir_name_hook($domain, $this->id);
         } elseif (Config::bool('domain_path')) {
             if (Config::bool('domain_in_mailbox')) {
@@ -438,7 +472,7 @@ class MailboxHandler extends PFAHandler {
         $fSubject = Config::lang('pSendmail_subject_text');
         $fBody = Config::read('welcome_text');
 
-        if (!smtp_mail($fTo, $fFrom, $fSubject, $fBody)) {
+        if (!smtp_mail($fTo, $fFrom, $fSubject, smtp_get_admin_password(), $fBody)) {
             $this->errormsg[] = Config::lang_f('pSendmail_result_error', $this->id);
             return false;
         }
@@ -450,13 +484,11 @@ class MailboxHandler extends PFAHandler {
     /**
      * Check if the user is creating a mailbox within the quota limits of the domain
      *
-     * @param Integer $quota - quota wanted for the mailbox
-     * @return Boolean - true if requested quota is OK, otherwise false
+     * @param int $quota - quota wanted for the mailbox
+     * @return boolean - true if requested quota is OK, otherwise false
+     * @todo merge with allowed_quota?
      */
-    # TODO: merge with allowed_quota?
     protected function check_quota($quota) {
-        $rval = false;
-
         if (!Config::bool('quota')) {
             return true; # enforcing quotas is disabled - just allow it
         }
@@ -464,10 +496,6 @@ class MailboxHandler extends PFAHandler {
         list(/*NULL*/, $domain) = explode('@', $this->id);
         $limit = get_domain_properties($domain);
 
-        if ($limit['maxquota'] == 0) {
-            $rval = true; # maxquota unlimited -> OK, but domain level quota could still be hit
-        }
-
         if (($limit['maxquota'] < 0) and ($quota < 0)) {
             return true; # maxquota and $quota are both disabled -> OK, no need for more checks
         }
@@ -478,12 +506,6 @@ class MailboxHandler extends PFAHandler {
 
         if ($limit['maxquota'] != 0 && $quota > $limit['maxquota']) {
             return false; # mailbox bigger than maxquota restriction (and maxquota != unlimited) -> not allowed, no more checks needed
-        } else {
-            $rval = true; # mailbox size looks OK, but domain level quota could still be hit
-        }
-
-        if (!$rval) {
-            return false; # over quota - no need to check domain_quota
         }
 
         # TODO: detailed error message ("domain quota exceeded", "mailbox quota too big" etc.) via flash_error? Or "available quota: xxx MB"?
@@ -495,11 +517,11 @@ class MailboxHandler extends PFAHandler {
             return false;
         } else {
             $table_mailbox = table_by_key('mailbox');
-            $query = "SELECT SUM(quota) FROM $table_mailbox WHERE domain = '" . escape_string($domain) . "'";
-            $query .= " AND username != '" . escape_string($this->id) . "'";
-            $result = db_query($query);
-            $row = db_row($result['result']);
-            $cur_quota_total = divide_quota($row[0]); # convert to MB
+            $query = "SELECT SUM(quota) as sum FROM $table_mailbox WHERE domain = ? AND username != ?";
+
+            $rows = db_query_all($query, array($domain, $this->id));
+
+            $cur_quota_total = divide_quota($rows[0]['sum']); # convert to MB
             if (($quota + $cur_quota_total) > $limit['quota']) {
                 $rval = false;
             } else {
@@ -514,9 +536,9 @@ class MailboxHandler extends PFAHandler {
     /**
      * Get allowed maximum quota for a mailbox
      *
-     * @param String $domain
-     * @param Integer $current_user_quota (in bytes)
-     * @return Integer allowed maximum quota (in MB)
+     * @param string $domain
+     * @param int $current_user_quota (in bytes)
+     * @return int allowed maximum quota (in MB)
      */
     protected function allowed_quota($domain, $current_user_quota) {
         if (!Config::bool('quota')) {
@@ -544,14 +566,14 @@ class MailboxHandler extends PFAHandler {
     /**
      * Called after a mailbox has been created or edited in the DBMS.
      *
-     * @return Boolean success/failure status
+     * @return boolean success/failure status
      */
     protected function mailbox_post_script() {
         if ($this->new) {
-            $cmd = Config::read('mailbox_postcreation_script');
+            $cmd = Config::read_string('mailbox_postcreation_script');
             $warnmsg = Config::Lang('mailbox_postcreate_failed');
         } else {
-            $cmd = Config::read('mailbox_postedit_script');
+            $cmd = Config::read_string('mailbox_postedit_script');
             $warnmsg = Config::Lang('mailbox_postedit_failed');
         }
 
@@ -573,7 +595,7 @@ class MailboxHandler extends PFAHandler {
         if ($quota <= 0) {
             $quota = 0;
         } # TODO: check if this is correct behaviour
-        $cmdarg4=escapeshellarg($quota);
+        $cmdarg4 = escapeshellarg("" . $quota);
         $command= "$cmd $cmdarg1 $cmdarg2 $cmdarg3 $cmdarg4";
         $retval=0;
         $output=array();
@@ -595,7 +617,7 @@ class MailboxHandler extends PFAHandler {
      * also adds a detailed error message to $this->errormsg[]
      */
     protected function mailbox_postdeletion() {
-        $cmd = Config::read('mailbox_postdeletion_script');
+        $cmd = Config::read_string('mailbox_postdeletion_script');
 
         if (empty($cmd)) {
             return true;
@@ -627,7 +649,7 @@ class MailboxHandler extends PFAHandler {
 
 
     /**
-     * Called by storemore() after a mailbox has been created.
+     * Called by postSave() after a mailbox has been created.
      * Immediately returns, unless configuration indicates
      * that one or more sub-folders should be created.
      *
@@ -640,7 +662,7 @@ class MailboxHandler extends PFAHandler {
      * Doesn't clean up, if only some of the folders could be
      * created.
      *
-     * @return Boolean TRUE if everything succeeds, FALSE on all errors
+     * @return boolean TRUE if everything succeeds, FALSE on all errors
      */
     protected function create_mailbox_subfolders() {
         $create_mailbox_subdirs = Config::read('create_mailbox_subdirs');
@@ -648,12 +670,17 @@ class MailboxHandler extends PFAHandler {
             return true;
         }
 
+        if (!function_exists('imap_open')) {
+            trigger_error('imap_open function not present; cannot create_mailbox_subdirs');
+            return false;
+        }
+
         if (!is_array($create_mailbox_subdirs)) {
             trigger_error('create_mailbox_subdirs must be an array', E_USER_ERROR);
             return false;
         }
 
-        $s_host = Config::read('create_mailbox_subdirs_host');
+        $s_host = Config::read_string('create_mailbox_subdirs_host');
         if (empty($s_host)) {
             trigger_error('An IMAP/POP server host ($CONF["create_mailbox_subdirs_host"]) must be configured, if sub-folders are to be created', E_USER_ERROR);
             return false;
@@ -673,7 +700,7 @@ class MailboxHandler extends PFAHandler {
         }
 
         $s_port='';
-        $create_mailbox_subdirs_hostport = Config::read('create_mailbox_subdirs_hostport');
+        $create_mailbox_subdirs_hostport = Config::read_string('create_mailbox_subdirs_hostport');
         if (!empty($create_mailbox_subdirs_hostport)) {
             $s_port = $create_mailbox_subdirs_hostport;
             if (intval($s_port)!=$s_port) {
@@ -693,7 +720,7 @@ class MailboxHandler extends PFAHandler {
             return false;
         }
 
-        $s_prefix = Config::read('create_mailbox_subdirs_prefix');
+        $s_prefix = Config::read_string('create_mailbox_subdirs_prefix');
         foreach ($create_mailbox_subdirs as $f) {
             $f='{'.$s_host.'}'.$s_prefix.$f;
             $res=imap_createmailbox($i, $f);
@@ -709,49 +736,6 @@ class MailboxHandler extends PFAHandler {
         return true;
     }
 
-
-    /********************************************************************************************************************
-         old functions - we'll see what happens to them
-         (at least they should use the *Handler functions instead of doing SQL)
-    /********************************************************************************************************************/
-
-    /**
-     * @return boolean true on success; false on failure
-     * @param string $old_password
-     * @param string $new_passwords
-     * @param bool $match = true
-     *
-     * All passwords need to be plain text; they'll be hashed appropriately
-     * as per the configuration in config.inc.php
-     */
-    public function change_pw($new_password, $old_password, $match = true) {
-        list(/*NULL*/, $domain) = explode('@', $this->id);
-
-        if ($match == true) {
-            if (!$this->login($this->id, $old_password)) {
-                db_log($domain, 'edit_password', "MATCH FAILURE: " . $this->id);
-                $this->errormsg[] = Config::Lang('pPassword_password_current_text_error');
-                return false;
-            }
-        }
-
-        $set = array(
-            'password' => pacrypt($new_password) ,
-        );
-
-        $result = db_update('mailbox', 'username', $this->id, $set);
-
-        if ($result != 1) {
-            db_log($domain, 'edit_password', "FAILURE: " . $this->id);
-            $this->errormsg[] = Config::lang('pEdit_mailbox_result_error');
-            return false;
-        }
-
-        db_log($domain, 'edit_password', $this->id);
-        return true;
-    }
-
-
     #TODO: more self explaining language strings!
 }
 
